package jp.sbcr.java_framework.auth;

import java.lang.reflect.Method;

import javax.servlet.http.HttpServletRequest;

import jp.sbcr.java_framework.auth.annotation.AuthenticationNotRequired;
import jp.sbcr.java_framework.auth.annotation.AuthenticationRequired;
import jp.sbcr.java_framework.auth.exception.AuthenticationException;

import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.seasar.framework.container.SingletonS2Container;

public class AuthenticationInterceptor implements
		MethodInterceptor {

	public Object invoke(MethodInvocation invocation)
			throws Throwable {

		processAuthenticationCheck(invocation);

		return invocation.proceed();
	}

	private void processAuthenticationCheck(
			MethodInvocation invocation) {

		Method method = invocation.getMethod();

		AuthenticationRequired required = method
				.getAnnotation(AuthenticationRequired.class);
		AuthenticationNotRequired notRequired = method
				.getAnnotation(AuthenticationNotRequired.class);

		if (required != null && notRequired != null) {
			throw new IllegalStateException(method + "には"
					+ "AuthenticationRequiredと"
					+ "AuthenticationNotRequiredの"
					+ "2つのアノテーションが記述されていますが、同時に2つは記述できません。");
		}

		HttpServletRequest req = getRequest();
		if (required != null && req.getUserPrincipal() == null) {
			throw new AuthenticationException();
		}
	}

	private static HttpServletRequest getRequest() {
		return SingletonS2Container
				.getComponent(HttpServletRequest.class);
	}

}
