package jp.sbcr.java_framework.auth;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jp.sbcr.java_framework.auth.annotation.AuthenticationNotRequired;
import jp.sbcr.java_framework.auth.annotation.AuthenticationRequired;
import jp.sbcr.java_framework.auth.exception.AuthenticationException;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.actions.DispatchAction;
import org.seasar.extension.unit.S2TestCase;
import org.seasar.framework.container.ExternalContext;
import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.external.servlet.HttpServletExternalContext;

public class AuthenticationInterceptorTest extends S2TestCase {

	private HttpServletRequest authRequest;

	private HttpServletRequest notAuthRequest;

	private TestAction testAction;

	private S2Container container;

	@Override
	public void setUp() {
		include("AuthenticationInterceptorTest.dicon");
	}

	@Override
	protected void setUpAfterBindFields() throws Throwable {
		super.setUpAfterBindFields();

		setUpNotAuthRequest();
	}

	private void setUpNotAuthRequest() {
		setUpRequest(notAuthRequest);
	}

	private void setUpAuthRequest() {
		setUpRequest(authRequest);
	}

	private void setUpRequest(HttpServletRequest request) {
		ExternalContext externalContext = new HttpServletExternalContext();
		externalContext.setApplication(getServletContext());
		externalContext.setRequest(request);
		externalContext.setResponse(getResponse());
		container.setExternalContext(externalContext);
	}

	public void testRequiredで未認証の時() {
		try {
			testAction.required(null, null, null, null);
		} catch (AuthenticationException success) {
			success.printStackTrace();
		}
	}

	public void testRequiredで認証済の時() {
		setUpAuthRequest();

		testAction.required(null, null, null, null);
	}

	public void testNotRequiredの時() {
		testAction.notRequired(null, null, null, null);
	}

	public void test2つアノテーションがある時() {
		try {
			testAction.both(null, null, null, null);
			fail();
		} catch (IllegalStateException success) {
			success.printStackTrace();
		}
	}

	public void testアノテーションがない時() {
		testAction.normal(null, null, null, null);
	}

	public static class TestAction extends DispatchAction {

		@AuthenticationRequired
		public ActionForward required(ActionMapping mapping,
				ActionForm form, HttpServletRequest request,
				HttpServletResponse response) {
			return null;
		}

		@AuthenticationNotRequired
		public ActionForward notRequired(ActionMapping mapping,
				ActionForm form, HttpServletRequest request,
				HttpServletResponse response) {
			return null;
		}

		@AuthenticationRequired
		@AuthenticationNotRequired
		public ActionForward both(ActionMapping mapping,
				ActionForm form, HttpServletRequest request,
				HttpServletResponse response) {
			return null;
		}

		public ActionForward normal(ActionMapping mapping,
				ActionForm form, HttpServletRequest request,
				HttpServletResponse response) {
			return null;
		}
	}
}
